(***********************************************************************)
(*                                                                     *)
(*                                CMSV                                 *)
(*                                                                     *)
(*                  Tom Hirschowitz, INRIA Rocquencourt                *)
(*                                                                     *)
(*  Copyright 2003 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU General Public License.                 *)
(*                                                                     *)
(***********************************************************************)

type 'a t

val add : 'a -> 'a t -> 'a t
val to_list : 'a t -> 'a list
val union : 'a t -> 'a t -> 'a t
val empty : 'a t
val remove : 'a -> 'a t -> 'a t
val diff : 'a t -> 'a t -> 'a t
val singleton : 'a -> 'a t

val length : 'a t -> int
val hd : 'a t -> 'a
val tl : 'a t -> 'a t
val nth : 'a t -> int -> 'a
val rev : 'a t -> 'a t
val append : 'a t -> 'a t -> 'a t
val rev_append : 'a t -> 'a t -> 'a t
val concat : 'a t t -> 'a t
val flatten : 'a t t -> 'a t
val iter : ('a -> unit) -> 'a t -> unit
val map : ('a -> 'b) -> 'a t -> 'b t
val rev_map : ('a -> 'b) -> 'a t -> 'b t
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
val fold_right : ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit
val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
val rev_map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
val fold_left2 : 
    ('a -> 'b -> 'c -> 'a) -> 'a -> 'b t -> 'c t -> 'a
val fold_right2 :
    ('a -> 'b -> 'c -> 'c) -> 'a t -> 'b t -> 'c -> 'c
val for_all : ('a -> bool) -> 'a t -> bool
val exists : ('a -> bool) -> 'a t -> bool
val for_all2 : ('a -> 'b -> bool) -> 'a t -> 'b t -> bool
val exists2 : ('a -> 'b -> bool) -> 'a t -> 'b t -> bool
val mem : 'a -> 'a t -> bool
val memq : 'a -> 'a t -> bool
val find : ('a -> bool) -> 'a t -> 'a
val filter : ('a -> bool) -> 'a t -> 'a t
val find_all : ('a -> bool) -> 'a t -> 'a t
val partition : ('a -> bool) -> 'a t -> 'a t * 'a t
val assoc : 'a -> ('a * 'b) t -> 'b
val assq : 'a -> ('a * 'b) t -> 'b
val mem_assoc : 'a -> ('a * 'b) t -> bool
val mem_assq : 'a -> ('a * 'b) t -> bool
val remove_assoc : 'a -> ('a * 'b) t -> ('a * 'b) t
val remove_assq : 'a -> ('a * 'b) t -> ('a * 'b) t
val split : ('a * 'b) t -> 'a t * 'b t
val combine : 'a t -> 'b t -> ('a * 'b) t
val sort : ('a -> 'a -> int) -> 'a t -> 'a t
val stable_sort : ('a -> 'a -> int) -> 'a t -> 'a t
val fast_sort : ('a -> 'a -> int) -> 'a t -> 'a t
val merge : ('a -> 'a -> int) -> 'a t -> 'a t -> 'a t
 
