(***********************************************************************)
(*                                                                     *)
(*                                CMSV                                 *)
(*                                                                     *)
(*                  Tom Hirschowitz, INRIA Rocquencourt                *)
(*                                                                     *)
(*  Copyright 2003 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU General Public License.                 *)
(*                                                                     *)
(***********************************************************************)

type name = Name.t

type degree = Strict | Safe

type ident = definition NamedLocations.identity

and env = (definition, Obj.t) NamedLocations.t

and body = env -> Obj.t

and t = definition list

and definition = 
    { label : ident ;
      mutable body : body ;
      mutable dependencies : dependencies ;
      size : size }

and dependencies = dependency list

and dependency = 
    { target : ident ;
      degree : degree }

and size = 
  | Known of int 
  | Unknown


(* Names defined by the output sequence, in order *)
val names : t -> Name.t list

(* Names deferred in the output sequence, no order *)
val deferred : t -> Name.t LSet.t

(* Dependencies manipulation *)
module Dependencies : sig
  val fold_left :
    ('a -> dependency -> 'a) -> 'a -> dependencies -> 'a
  val cons : dependency -> dependencies -> dependencies
  val empty : dependencies
end

(* Degree manipulation *)
module Degree : sig
  val min : degree -> degree -> degree
end

(* Graphs *)
module MixGraph : Graph.GRAPH with 
type label = degree
and type node_description = definition

val graph_of_t : t -> MixGraph.t

(*********************************************************)
(*                   Mixin operators                     *)
(*********************************************************)

(* Close a mixin module and return the resulting record. *)
val close : t -> Obj.t

(* Compose two mixin modules *)
val add : t -> t -> t

(* Freeze a definition in a mixin module *)
val freeze : Name.t -> t -> t


(* Debugging *)
module Print : sig 
val f : Format.formatter -> t -> unit
val print_degree : Format.formatter -> degree -> unit
val print_label : Format.formatter -> ident -> unit
end
