(***********************************************************************)
(*                                                                     *)
(*                                CMSV                                 *)
(*                                                                     *)
(*                  Tom Hirschowitz, INRIA Rocquencourt                *)
(*                                                                     *)
(*  Copyright 2003 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU General Public License.                 *)
(*                                                                     *)
(***********************************************************************)

exception Undefined_component of Name.t
exception Undefined_local_component 
exception Already_defined_component of Name.t
exception Already_defined_local_component

type 'anonymous identity = Late of Name.t | Early of 'anonymous
type ('a, 'o) t = ('a identity * 'o) list 

let empty = []

let list_of_named env = 
  List.rev
    (List.fold_left
       (fun  yet (ident, obj) -> obj :: yet)
       []
       env)

let find_exported name env = 
  try
    List.assoc (Late name) env
  with
    Not_found -> raise (Undefined_component name)

let rec find_local body env = 
match env with
| [] -> raise Undefined_local_component
| (Early body', obj) :: rest when body == body' -> obj
| _ :: rest -> find_local body rest

let extend_exported env name obj = 
  if 
    List.mem_assoc (Late name) env
  then 
    raise (Already_defined_component name)
  else
    (Late name, obj) :: env

let override_exported env name obj = 
  (Late name, obj) :: env

let extend_local env body obj =
  try 
    let _ = find_local body env in
    raise Already_defined_local_component
  with Undefined_local_component ->
    (Early body, obj) :: env

let override_local env def obj = 
  (Early def, obj) :: env

let find id env = match id with
| Late name -> find_exported name env
| Early body -> find_local body env

let extend env id obj = match id with
| Late name -> extend_exported env name obj
| Early body -> extend_local env body obj

let override env id obj = match id with
| Late name -> override_exported env name obj
| Early body -> override_local env body obj

(* Compose a (ident, 'a) t with a (ident, ident) t.
   Preserve env1 outside of the domain of env2. 
   Start with env1, and for each modification id1 -> id2 in
   env2, override env1 with (id1 -> (env1 id2)). 
   There must be no garbage in env2, ie no ident
   is allowed to be bound twice. 
   The environment env1 is expected to provide all the 
   components required by env2. *)
let compose env1 env2 = 
  List.fold_left
    (fun yet (ident, ident') ->
      try
	let obj' = find ident' env1 in
	override yet ident obj'
      with _ -> yet)
    env1
    env2

(* Make an environment mapping name to (env (Early def)), 
   overriding previous behavior on name.
   To be used carefully... *)
let unfreeze name def env = 
  let obj = find_local def env in
  let env' = override_exported env name obj in
  env'


