(***********************************************************************)
(*                                                                     *)
(*                                CMSV                                 *)
(*                                                                     *)
(*                  Tom Hirschowitz, INRIA Rocquencourt                *)
(*                                                                     *)
(*  Copyright 2003 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU General Public License.                 *)
(*                                                                     *)
(***********************************************************************)

exception Undefined_component of Name.t
exception Undefined_local_component 
exception Already_defined_component of Name.t
exception Already_defined_local_component

type 'anonymous identity = Late of Name.t | Early of 'anonymous
type ('a, 'o) t = ('a identity * 'o) list 

val empty : ('a, 'o) t

val list_of_named : ('a, 'o) t -> 'o list

val find_exported : Name.t -> ('a, 'o) t -> 'o
val extend_exported : ('a, 'o) t -> Name.t -> 'o -> ('a, 'o) t
val override_exported : ('a, 'o) t -> Name.t -> 'o -> ('a, 'o) t
val find_local : 'a -> ('a, 'o) t -> 'o
val extend_local : ('a, 'o) t -> 'a -> 'o -> ('a, 'o) t
val override_local : ('a, 'o) t -> 'a -> 'o -> ('a, 'o) t

val find : 'a identity -> ('a, 'o) t -> 'o
val extend : ('a, 'o) t -> 'a identity -> 'o -> ('a, 'o) t

(* Compose a (ident, 'a) t with a (ident, ident) t.
   Preserve env1 outside of the domain of env2. 
   Start with env1, and for each modification id1 -> id2 in
   env2, override env1 with (id1 -> (env1 id2)). 
   There must be no garbage in env2, ie no ident
   is allowed to be bound twice. 
   The environment env1 is expected to provide all the 
   components required by env2. *)
val compose : ('a, 'o) t -> ('a, 'a identity) t -> ('a, 'o) t

(* Make an environment mapping name to (env (Early def)), 
   overriding previous behavior on name.
   To be used carefully... *)
val unfreeze : Name.t -> 'a -> ('a, 'o) t -> ('a, 'o) t
